package cz.nic.tablexia.game;

import cz.nic.tablexia.Tablexia.ChangeScreenEvent;
import cz.nic.tablexia.TablexiaApplication.ScreenTransaction;
import cz.nic.tablexia.bus.ApplicationBus;
import cz.nic.tablexia.bus.ApplicationBus.ApplicationEvent;
import cz.nic.tablexia.game.games.in_the_darkness.InTheDarknessScreen;
import cz.nic.tablexia.game.games.kidnapping.KidnappingScreen;
import cz.nic.tablexia.game.games.night_watch.NightWatchScreen;
import cz.nic.tablexia.game.games.pursuit.PursuitScreen;
import cz.nic.tablexia.game.games.robbery.RobberyScreen;
import cz.nic.tablexia.game.games.shooting_range.ShootingRangeScreen;
import cz.nic.tablexia.loader.application.ApplicationTextManager;
import cz.nic.tablexia.menu.IMenuItem;


public enum GameDefinition implements ApplicationEvent, IMenuItem {

	ROBBERY				("game_robbery_title",          RobberyScreen.class,        true),
    PURSUIT             ("game_pursuit_title",          PursuitScreen.class,        true),
    KIDNAPPING          ("game_kidnapping_title",       KidnappingScreen.class,     true),
    NIGHT_WATCH         ("game_night_watch_title",      NightWatchScreen.class,     true),
    SHOOTING_RANGE      ("game_shooting_range_title",   ShootingRangeScreen.class,  true),
    IN_THE_DARKNESS     ("game_in_the_darkness_title",  InTheDarknessScreen.class,  true);


	private String 										menuTextKey;
	private Class<? extends AbstractTablexiaGame<?>> 	screenClass;
	private boolean 									isCloseMenu;

    public static GameDefinition[] getActiveGames(){
        return new GameDefinition[]{ROBBERY, PURSUIT, KIDNAPPING, NIGHT_WATCH, SHOOTING_RANGE, IN_THE_DARKNESS};
    }

	private GameDefinition(String nameResource, Class<? extends AbstractTablexiaGame<?>> screenClass, boolean isCloseMenu) {
		this.menuTextKey = nameResource;
		this.screenClass = screenClass;
		this.isCloseMenu = isCloseMenu;
	}

	@Override
	public String getTitle() {
		return ApplicationTextManager.getInstance().getResult().get(menuTextKey);
	}

	@Override
	public void performAction() {
		ApplicationBus.getInstance().publishAsync(this);
		ApplicationBus.getInstance().publishAsync(new ChangeScreenEvent(screenClass, ScreenTransaction.FADE));
	}

	@Override
	public boolean isCloseMenu() {
		return isCloseMenu;
	}

    public static GameDefinition getGameDefinitionForClass(Class<? extends AbstractTablexiaGame<?>> screenClass) {
        for (GameDefinition gameDefinition : GameDefinition.values()) {
            if (gameDefinition.screenClass.equals(screenClass)) {
                return gameDefinition;
            }
        }
        return null;
    }

}
