package cz.nic.tablexia.game.games.robbery;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.InputMultiplexer;
import com.badlogic.gdx.InputProcessor;
import com.badlogic.gdx.audio.Sound;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.BitmapFont;

import java.util.List;

import cz.nic.tablexia.game.AbstractTablexiaGame;
import cz.nic.tablexia.game.games.robbery.assets.RobberyAssets;
import cz.nic.tablexia.game.games.robbery.creature.CreatureFactory;
import cz.nic.tablexia.game.games.robbery.rules.GameRule;
import cz.nic.tablexia.game.games.robbery.rules.GameRulesDefinition;
import cz.nic.tablexia.loader.application.ApplicationTextureManager;
import cz.nic.tablexia.screen.AbstractTablexiaScreen;

public class RobberyScreen extends AbstractTablexiaGame<GameRule> {
	
	private static int			CREATURES_COUNT 				= 50;
	private static int 			THIEVES_COUNT					= 8;
	
	private static RobberyScreen instance			= null;
	
	private InputMultiplexer						inputMultiplexer;
	private AbstractTablexiaScreen<Void> 			actualScreen;
	
	private boolean isRootScreenVisible;
	private boolean isComponentScreenVisible;
	
	
//////////////////////////// SCREEN LOADERS
	
	@Override
	protected void prepareGameTextureAssetNames(List<String> textureFileNames) {
		textureFileNames.add(ApplicationTextureManager.UniversalAssets.BACKGROUND_WOODEN);
		
		textureFileNames.add(RobberyAssets.SCREEN_RULEMESSAGE_PAPER_S1);
		textureFileNames.add(RobberyAssets.SCREEN_BANK_S1);
		textureFileNames.add(RobberyAssets.SCREEN_NEWSSTAND_S5);
		textureFileNames.add(RobberyAssets.SCREEN_NEWSSTAND_BOTTOM_S5);
		textureFileNames.add(RobberyAssets.SCREEN_BACKGROUND_S5);
		
		textureFileNames.add(RobberyAssets.INFOITEM_ARRESTED);
		textureFileNames.add(RobberyAssets.INFOITEM_INNOCENCE);
		textureFileNames.add(RobberyAssets.INFOITEM_ALARM);
		
		
		textureFileNames.add(RobberyAssets.CREATURE_BACKGROUND_TEXTURE);
		
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_BEADS_BLUE);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_BEADS_GREY);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_BEADS_RED);
		
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_SCARF_GREEN);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_SCARF_PURPLE);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_SCARF_YELLOW);
		
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_SCARF_BLUE);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_SCARF_BROWN);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_SCARF_YELLOW);
		
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_TIE_GREEN);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_TIE_GREY);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_TIE_RED);
		
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_PANTS_BLUE);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_PANTS_BROWN);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_PANTS_GREEN);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_PANTS_GREY);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_PANTS_PURPLE);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_PANTS_RED);
		
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_SKIRT_BLUE);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_SKIRT_GREEN);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_SKIRT_GREY);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_SKIRT_PURPLE);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_SKIRT_RED);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_SKIRT_YELLOW);
		
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_PANTS_BLUE);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_PANTS_BROWN);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_PANTS_GREEN);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_PANTS_GREY);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_PANTS_RED);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_PANTS_YELLOW);
		
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_SHORT_BLUE);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_SHORT_GREEN);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_SHORT_GREY);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_SHORT_PURPLE);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_SHORT_RED);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_SHORT_YELLOW);
		
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_HAT_BLUE);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_HAT_GREEN);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_HAT_GREY);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_HAT_PURPLE);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_HAT_RED);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_HAT_YELLOW);
		
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_HAT_BROWN);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_HAT_GREEN);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_HAT_GREY);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_HAT_PURPLE);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_HAT_RED);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_HAT_YELLOW);
		
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_COAT_BROWN);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_COAT_GREEN);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_COAT_GREY);
		
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_SHIRT_BLUE);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_SHIRT_GREEN);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_SHIRT_GREY);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_SHIRT_PURPLE);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_SHIRT_RED);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_SHIRT_YELLOW);
		
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_SWEATER_BROWN);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_SWEATER_GREEN);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_SWEATER_GREY);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_SWEATER_PURPLE);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_SWEATER_RED);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_SWEATER_YELLOW);
		
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_VEST_BLUE);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_VEST_BROWN);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_VEST_GREEN);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_VEST_GREY);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_VEST_PURPLE);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_VEST_YELLOW);
		
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_COAT_BLUE);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_COAT_BROWN);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_COAT_GREY);
		
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_SHIRT_BLUE);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_SHIRT_BROWN);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_SHIRT_GREEN);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_SHIRT_GREY);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_SHIRT_RED);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_SHIRT_YELLOW);
		
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_SWEATER_BLUE);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_SWEATER_BROWN);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_SWEATER_GREEN);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_SWEATER_GREY);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_SWEATER_RED);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_SWEATER_YELLOW);
		
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_VEST_BLUE);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_VEST_GREEN);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_VEST_GREY);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_VEST_PURPLE);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_VEST_RED);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_VEST_YELLOW);
		
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_GLASSES);
		
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_GLASSES);
		
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_BRAIDHAIR_BLACK);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_BRAIDHAIR_BROWN);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_BRAIDHAIR_YELLOW);
		
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_LONGHAIR_BLACK);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_LONGHAIR_BROWN);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_LONGHAIR_YELLOW);
		
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_HAIR_BLACK);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_HAIR_BROWN);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_HAIR_YELLOW);
		
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_HEAD_1);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_F_HEAD_2);
		
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_HEAD_1);
		textureFileNames.add(RobberyAssets.ATTRIBUTE_M_HEAD_2);
	}
	
	@Override
	protected void prepareScreenSoundAssetNames(List<String> soundsFileNames) {
		soundsFileNames.add(RobberyAssets.SOUND_ALARM);
		soundsFileNames.add(RobberyAssets.SOUND_ERROR);
		soundsFileNames.add(RobberyAssets.SOUND_CHAINS);
	}
	
	@Override
	protected String prepareScreenTextResourcesAssetName() {
		return RobberyAssets.TEXT_PATH;
	}

	@Override
	protected GameRule prepareGameData() {
		GameRulesDefinition rule = GameRulesDefinition.getRandomGameRuleForDifficulty(getGameDifficulty(), getRandomAccess().getRandom());
        GameRule gameRule = rule.getGameRuleInstance(getRandomAccess(), CREATURES_COUNT, THIEVES_COUNT);
        gameRule.generateCreatures();

        printScreenInfo("Game Rule", gameRule.getGameRuleDefinition().name());
        
		return gameRule;
	}
	
	
//////////////////////////// INPUT PROCESSOR
	
	
	@Override
	public InputProcessor getInputProcessor() {
		return inputMultiplexer;
	}
	
	
//////////////////////////// SCREEN LIFECYCLE
	
	public RobberyScreen() {
		instance = this;
		isRootScreenVisible = false;
		isComponentScreenVisible = false;
		inputMultiplexer = new InputMultiplexer();
        inputMultiplexer.addProcessor(getStage());
	}

	@Override
	protected void gameLoaded() {
		showRuleScreen(false);
	}
	
	@Override
	protected void screenVisible() {
		isRootScreenVisible = true;
		performComponentScreenVisible();
	}
	
	@Override
	public void screenResized(int width, int height) {
		if (actualScreen != null) {
			actualScreen.resize(width, height);
		}
	}
	
	@Override
	public void screenRender(float delta) {
		if (actualScreen != null) {
			actualScreen.render(delta);
		}
	}
	
	@Override
	protected void screenDisposed() {
		actualScreen.dispose();
		CreatureFactory.getInstance().dispose();
		instance = null;
	}
	
	
//////////////////////////// COMPONENT SCREENS
	
	private void showScreen(final AbstractTablexiaScreen<Void> newScreen, boolean transaction) {
		isComponentScreenVisible = false;
		if (transaction) {
            performHideTransaction(new Runnable() {

                @Override
                public void run() {
                    processNewScreen(newScreen);
                    performShowTransaction(new Runnable() {
                        @Override
                        public void run() {
                            isComponentScreenVisible = true;
                            performComponentScreenVisible();
                        }
                    });
                }
            });
		} else {
			processNewScreen(newScreen);
			isComponentScreenVisible = true;
			performComponentScreenVisible();
		}
	}
	
	private void performComponentScreenVisible() {
		if (isRootScreenVisible && isComponentScreenVisible) {
			isComponentScreenVisible = false;
			actualScreen.performScreenVisible();
		}
	}
	
	private void processNewScreen(AbstractTablexiaScreen<Void> newScreen) {
		AbstractTablexiaScreen<Void> lastScreen = actualScreen;
		actualScreen = newScreen;


		inputMultiplexer.addProcessor(newScreen.getInputProcessor());
		newScreen.show();
		if (lastScreen != null) {			
			inputMultiplexer.removeProcessor(lastScreen.getInputProcessor());
			lastScreen.dispose();
		}
	}
	
	private void showGameScreen(final boolean transaction) {
		Gdx.app.postRunnable(new Runnable() {
			
			@Override
			public void run() {
				showScreen(new GameScreen(), transaction);
			}
		});
	}
	
	private void showRuleScreen(final boolean transaction) {
		Gdx.app.postRunnable(new Runnable() {
			
			@Override
			public void run() {
				showScreen(new RuleScreen(), transaction);
			}
		});
	}
	
	
//////////////////////////// LOADERS ACCESS
	
	public static Texture getRobberyTexture(String textureName) {
		if (instance != null) {
			return instance.getTexture(textureName);
		}
		return null;
	}
	
	public static Sound getRobberySound(String soundName) {
		if (instance != null) {
			return instance.getSound(soundName);
		}
		return null;
	}
	
	public static String getRobberyText(String key) {
		if (instance != null) {
			return instance.getText(key);
		}
		return null;
	}
	
	public static String getRobberyFormatedText(String key, Object... args) {
		if (instance != null) {
			return instance.getFormattedText(key, args);
		}
		return null;
	}

    public static BitmapFont getRobberyRegularFont() {
        if (instance != null) {
            return instance.getDefaultRegularFont();
        }
        return null;
    }

    public static BitmapFont getRobberyBoldFont() {
        if (instance != null) {
            return instance.getDefaultBoldFont();
        }
        return null;
    }
	
	public static GameRule getRobberyData() {
		if (instance != null) {
			return instance.getData();
		}
		return null;
	}
	
	static void showRobberyRuleScreen() {
		instance.showRuleScreen(true);
	}
	
	static void showRobberyGameScreen() {
		instance.showGameScreen(true);
	}

    static void gameRobberyComplete() {
        instance.gameComplete(2);
    }
}
