package cz.nic.tablexia.game.games.robbery;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.Texture.TextureFilter;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.InputListener;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.Label.LabelStyle;

import cz.nic.tablexia.game.games.robbery.assets.RobberyAssets;
import cz.nic.tablexia.loader.application.ApplicationTextureManager;
import cz.nic.tablexia.screen.AbstractTablexiaScreen;

public class RuleScreen extends AbstractTablexiaScreen<Void> {

	private static final 	float TEXT_PAPER_POSITION_Y_RATIO	= 1f/2;
	private static final 	float TEXT_PAPER_WIDTH_RATIO 		= 3f/5;
	private static final 	float TEXT_PAPER_HEIGHT_RATIO 		= 1f/5;
	private static final 	Color TEXT_COLOR 					= Color.BLACK;
	
	private 				RuleBackground 	background;
	private 				RulePaper 		paper;
	private 				Label 			label;
	
	private 				boolean 		loadingComplete;

	private class RuleBackground extends Actor {
		
		private Texture backgroundTexture;

		public RuleBackground() {
			backgroundTexture = RobberyScreen.getRobberyTexture(ApplicationTextureManager.UniversalAssets.BACKGROUND_WOODEN);
			backgroundTexture.setFilter(TextureFilter.Linear, TextureFilter.Linear);
		}
		
		@Override
		public void draw(Batch batch, float parentAlpha) {
			batch.disableBlending();
			batch.draw(backgroundTexture, 0, 0, getWidth(), getHeight());
			batch.enableBlending();
		}
		
	}
	
	private class RulePaper extends Actor {
		
		private static final float PAPER_HEIGHT_RATIO = 9f/10;
		
		private Texture paperTexture;

		public RulePaper() {
			paperTexture = RobberyScreen.getRobberyTexture(RobberyAssets.SCREEN_RULEMESSAGE_PAPER_S1);
			paperTexture.setFilter(TextureFilter.Linear, TextureFilter.Linear);
		}
		
		@Override
		public void setSize(float width, float height) {
			float smallerScreenSize = height < width ? height : width;
			float smallerPaperSize = height < width ? paperTexture.getHeight() : paperTexture.getWidth();
			float paperRatio = (smallerScreenSize * PAPER_HEIGHT_RATIO) / smallerPaperSize;
			super.setSize(paperTexture.getWidth() * paperRatio, paperTexture.getHeight() * paperRatio);
			super.setPosition((width / 2) - (getWidth() / 2), (height / 2) - (getHeight() / 2));
		}
		
		@Override
		public void draw(Batch batch, float parentAlpha) {
			batch.draw(paperTexture, getX(), getY(), getWidth(), getHeight());
		}
		
	}
	
	public RuleScreen() {
		loadingComplete = false;
	}
	
	
//////////////////////////// SCREEN LIFECYCLE
	
	@Override
	protected void screenLoaded() {
		background = new RuleBackground();
		paper = new RulePaper();
		paper.addListener(new InputListener() {
			
			@Override
			public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {
				RobberyScreen.showRobberyGameScreen();
				return true;
			}
			
		});
		label = new Label(RobberyScreen.getRobberyData().getRuleMessageText(), new LabelStyle(RobberyScreen.getRobberyRegularFont(), TEXT_COLOR));
		label.setWrap(true);
		
		getStage().addActor(background);
		getStage().addActor(paper);
		getStage().addActor(label);
		
		loadingComplete = true;

        screenResized((int)getStage().getWidth(), (int)getStage().getHeight());
	}
	
	@Override
	public void screenResized(int width, int height) {
		if (loadingComplete) {
			background.setSize(width, height);
			paper.setSize(width, height);
			label.setSize(paper.getWidth() * TEXT_PAPER_WIDTH_RATIO, paper.getHeight() * TEXT_PAPER_HEIGHT_RATIO);
			label.setPosition((width / 2) - (label.getWidth() / 2), (height * TEXT_PAPER_POSITION_Y_RATIO) - (label.getHeight() / 2));
		}
	}
}
