package cz.nic.tablexia.loader.application;

import com.badlogic.gdx.assets.AssetManager;
import com.badlogic.gdx.assets.loaders.FileHandleResolver;
import com.badlogic.gdx.assets.loaders.resolvers.InternalFileHandleResolver;
import com.badlogic.gdx.graphics.Texture.TextureFilter;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.freetype.FreeTypeFontGenerator;
import com.badlogic.gdx.graphics.g2d.freetype.FreeTypeFontGeneratorLoader;
import com.badlogic.gdx.graphics.g2d.freetype.FreetypeFontLoader;
import com.badlogic.gdx.graphics.g2d.freetype.FreetypeFontLoader.FreeTypeFontLoaderParameter;

import cz.nic.tablexia.loader.IApplicationLoader;

/**
 * Font loader and manager class for application context
 * 
 * @author Matyáš Latner
 *
 */
public class ApplicationFontManager extends AssetManager implements IApplicationLoader {
	
	private static final int 		FONT_SIZE 				= 20;
	private static final String 	CZECH_CHARACTERS 		= "ěřťůšďžčňĚŘŤŮŠĎŽČŇ";
	private static final String     FONT_PATH               = "font/";
	public static final String 		FONT_ROBOTO_REGULAR		= FONT_PATH + "Roboto-Regular.ttf";
	public static final String 		FONT_ROBOTO_BOLD 		= FONT_PATH + "Roboto-Bold.ttf";

    public static final String      APPLICATION_DEFAULT_FONT_REGULAR    = FONT_ROBOTO_REGULAR;
    public static final String      APPLICATION_DEFAULT_FONT_BOLD       = FONT_ROBOTO_BOLD;
	
	private static ApplicationFontManager instance;
	
	private ApplicationFontManager() {
		FileHandleResolver resolver = new InternalFileHandleResolver();
		setLoader(FreeTypeFontGenerator.class, new FreeTypeFontGeneratorLoader(resolver));
		setLoader(BitmapFont.class, ".ttf", new FreetypeFontLoader(resolver));
	}
	
	public static ApplicationFontManager getInstance() {
		if (instance == null) {
			instance = new ApplicationFontManager();
		}
		return instance;
	}
	
	@Override
	public synchronized void dispose() {
		super.dispose();
		instance = null;
	}
	
	public void load() {
		loadFont(FONT_ROBOTO_REGULAR);
		loadFont(FONT_ROBOTO_BOLD);
	}
	
	private void loadFont(String fontFile) {
		FreeTypeFontLoaderParameter fontParams = new FreeTypeFontLoaderParameter();
		fontParams.fontFileName = fontFile;
		fontParams.fontParameters.size = FONT_SIZE;
		fontParams.fontParameters.characters = FreeTypeFontGenerator.DEFAULT_CHARS + CZECH_CHARACTERS;
		fontParams.fontParameters.minFilter = TextureFilter.Linear;
		fontParams.fontParameters.magFilter = TextureFilter.Linear;
		load(fontFile, BitmapFont.class, fontParams);
	}

    public BitmapFont getFont(String fontName) {
        return get(fontName, BitmapFont.class);
    }

    public BitmapFont getDefaultApplicationRegularFont() {
        return getFont(APPLICATION_DEFAULT_FONT_REGULAR);
    }

    public BitmapFont getDefaultApplicationBoldFont() {
        return getFont(APPLICATION_DEFAULT_FONT_BOLD);
    }
}