package cz.nic.tablexia.loader.application;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.utils.I18NBundle;
import com.badlogic.gdx.utils.async.AsyncTask;

import java.util.Locale;

import cz.nic.tablexia.loader.IApplicationLoader;
import cz.nic.tablexia.loader.TablexiaDataManager;

/**
 * Texture loader and manager for application context.
 * 
 * @author Matyáš Latner
 *
 */
public class ApplicationTextManager extends TablexiaDataManager<I18NBundle> implements IApplicationLoader {

    public static final class ApplicationTextsAssets {
        public static final String VICTORYSCREEN_BUTTON_REPLAY      = "victoryscreen_button_replay";
        public static final String VICTORYSCREEN_BUTTON_CHANGEGAME  = "victoryscreen_button_changegame";
    }
	
	private static final String APPLICATION_TEXT_RESOURCE_FILE = "text/application/application";

    public static final String LANGUAGE_SYSTEM = "language_system";
    public static final String LANGUAGE_CZECH  = "language_czech";
    public static final String LANGUAGE_SLOVAK = "language_slovak";

	private static class ApplicationTextLoader implements AsyncTask<I18NBundle> {

		private String textResourceFileName;
		private Locale locale;

		public ApplicationTextLoader(String textResourceFileName, Locale locale) {
			this.textResourceFileName = textResourceFileName;
			this.locale = locale;
		}

		@Override
		public I18NBundle call() throws Exception {
			FileHandle baseFileHandle = Gdx.files.internal(textResourceFileName);
			return I18NBundle.createBundle(baseFileHandle, locale);
		}
	}

	private static ApplicationTextManager instance;
	
	private ApplicationTextManager() {}
	
	public static ApplicationTextManager getInstance() {
		if (instance == null) {
			instance = new ApplicationTextManager();
		}
		return instance;
	}
	
	public void load(Locale locale) {
        setAsyncTask(new ApplicationTextLoader(APPLICATION_TEXT_RESOURCE_FILE, locale));
	}

    public String getText(String textKey) {
        return getResult().get(textKey);
    }

    public String getFormattedText(String textKey, Object... args) {
        return getResult().format(textKey, args);
    }
	
	@Override
	public void dispose() {
		super.dispose();
		instance = null;
	}
	
}