package cz.nic.tablexia.menu;

import java.util.ArrayList;
import java.util.List;

import cz.nic.tablexia.Tablexia;
import cz.nic.tablexia.TablexiaApplication;
import cz.nic.tablexia.TablexiaSettings;
import cz.nic.tablexia.bus.ApplicationBus;
import cz.nic.tablexia.bus.ApplicationBus.ApplicationEvent;
import cz.nic.tablexia.game.GameDefinition;
import cz.nic.tablexia.loader.application.ApplicationTextManager;
import cz.nic.tablexia.screen.about.AboutScreen;
import cz.nic.tablexia.screen.encyclopedia.EncyclopediaScreen;
import cz.nic.tablexia.screen.halloffame.HallOfFameScreen;
import cz.nic.tablexia.screen.preferences.PreferencesScreen;
import cz.nic.tablexia.screen.statistics.StatisticsScreen;
import cz.nic.tablexia.screen.viewport.ViewportDebugScreen;


public enum MainMenuDefinition implements IMenuItem {

	GAMES				(false, "mainmenu_games", 			GameDefinition.values(), false, null), // TODO open menu event
	HALL_OF_FAME		(false, "mainmenu_halloffame", 	    null, true, new Tablexia.ChangeScreenEvent(HallOfFameScreen.class, TablexiaApplication.ScreenTransaction.FADE)),
	STATISTICS			(false, "mainmenu_statistics", 	    null, true, new Tablexia.ChangeScreenEvent(StatisticsScreen.class, TablexiaApplication.ScreenTransaction.FADE)),
	ENCYCLOPEDIA		(false, "mainmenu_encyclopedia", 	null, true, new Tablexia.ChangeScreenEvent(EncyclopediaScreen.class, TablexiaApplication.ScreenTransaction.FADE)),
    PREFERENCES	        (false, "mainmenu_preferences",	    null, true, new Tablexia.ChangeScreenEvent(PreferencesScreen.class, TablexiaApplication.ScreenTransaction.FADE)),
	ABOUT_APPLICATION	(false, "mainmenu_about", 			null, true, new Tablexia.ChangeScreenEvent(AboutScreen.class, TablexiaApplication.ScreenTransaction.FADE)),
	LOGOUT				(false, "mainmenu_logout", 		    null, true, null), // TODO specify event
    SCREEN_DEBUG    	(true,  "mainmenu_screendebug",     null, true, new Tablexia.ChangeScreenEvent(ViewportDebugScreen.class, TablexiaApplication.ScreenTransaction.FADE));

    private boolean debug;
    private String menuTextKey;
	private IMenuItem[] submenu;
	private boolean isCloseMenu;
	private ApplicationEvent action;

	private MainMenuDefinition(boolean debug, String nameResource, IMenuItem[] submenu, boolean isCloseMenu, ApplicationEvent action) {
        this.debug = debug;
        this.menuTextKey = nameResource;
		this.submenu = submenu;
		this.isCloseMenu = isCloseMenu;
		this.action = action;
	}

    public ApplicationEvent getAction() {
		return action;
	}

	@Override
	public String getTitle() {
		return ApplicationTextManager.getInstance().getText(menuTextKey);
	}

	@Override
	public void performAction() {
		ApplicationBus.getInstance().publishAsync(this.getAction());
	}

	public IMenuItem[] getSubmenu() {
		return submenu;
	}

	@Override
	public boolean isCloseMenu() {
		return isCloseMenu;
	}

    public static List<MainMenuDefinition> getItemsForMenu() {
        List<MainMenuDefinition> mainMenu = new ArrayList<MainMenuDefinition>();
        for (MainMenuDefinition mainMenuDefinition : MainMenuDefinition.values()) {
            if (TablexiaSettings.getInstance().isDebug() || !mainMenuDefinition.debug) {
                mainMenu.add(mainMenuDefinition);
            }
        }
        return mainMenu;
    }
}
