package cz.nic.tablexia.screen;

import net.engio.mbassy.listener.Handler;

import cz.nic.tablexia.Tablexia.ChangeScreenEvent;
import cz.nic.tablexia.TablexiaApplication.ScreenTransaction;
import cz.nic.tablexia.bus.ApplicationBus;
import cz.nic.tablexia.menu.MainMenuDefinition;
import cz.nic.tablexia.screen.about.AboutScreen;
import cz.nic.tablexia.screen.encyclopedia.EncyclopediaScreen;
import cz.nic.tablexia.screen.gamemenu.GameMenuScreen;
import cz.nic.tablexia.screen.halloffame.HallOfFameScreen;
import cz.nic.tablexia.screen.loader.LoaderScreen;
import cz.nic.tablexia.screen.preferences.PreferencesScreen;
import cz.nic.tablexia.screen.statistics.StatisticsScreen;

public enum ScreenDefinition {
    LOADER(LoaderScreen.class, null),
    GAME_MENU(GameMenuScreen.class, null),
    HALL_OF_FAME(HallOfFameScreen.class, MainMenuDefinition.HALL_OF_FAME),
    STATISTICS(StatisticsScreen.class, MainMenuDefinition.STATISTICS),
    ENCYCLOPEDIA(EncyclopediaScreen.class, MainMenuDefinition.ENCYCLOPEDIA),
    PREFERENCES(PreferencesScreen.class, MainMenuDefinition.PREFERENCES),
    ABOUT_APPLICATION(AboutScreen.class, MainMenuDefinition.ABOUT_APPLICATION);

    private        Class<? extends AbstractTablexiaScreen<?>> screenClass;
    private        MainMenuDefinition                         mainMenuDefinition;
    private static Object                                     messageHandler;

    private ScreenDefinition(Class<? extends AbstractTablexiaScreen<?>> screenClass, MainMenuDefinition mainMenuDefinition) {
        this.screenClass = screenClass;
        this.mainMenuDefinition = mainMenuDefinition;
    }

    public MainMenuDefinition getMainMenuDefinition() {
        return mainMenuDefinition;
    }

    public Class<? extends AbstractTablexiaScreen<?>> getScreenClass() {
        return screenClass;
    }

    public void performMenuAction() {
        ApplicationBus.getInstance().publishAsync(new ChangeScreenEvent(screenClass, ScreenTransaction.FADE));
    }

    public static ScreenDefinition getScreenDefinitionForMainMenuDefinition(MainMenuDefinition mainMenuDefinition) {
        for (ScreenDefinition screenDefinition : ScreenDefinition.values()) {
            if (screenDefinition.getMainMenuDefinition() == mainMenuDefinition) {
                return screenDefinition;
            }
        }
        return null;
    }

    public static void initBus() {
        messageHandler = new Object() {

            @Handler
            public void handleChangeScreenEvent(MainMenuDefinition mainMenuDefinition) {
                ScreenDefinition.getScreenDefinitionForMainMenuDefinition(mainMenuDefinition).performMenuAction();
            }
        };
        ApplicationBus.getInstance().subscribe(messageHandler);
    }
}
