package cz.nic.tablexia.screen.gamemenu.pages;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.InputListener;
import com.badlogic.gdx.scenes.scene2d.actions.AlphaAction;
import com.badlogic.gdx.scenes.scene2d.actions.MoveToAction;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.Stack;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;

import cz.nic.tablexia.TablexiaSettings;
import cz.nic.tablexia.game.GameDefinition;
import cz.nic.tablexia.game.difficulty.GameDifficulty;
import cz.nic.tablexia.screen.AbstractTablexiaScreen;
import cz.nic.tablexia.screen.gamemenu.GameMenuAssets;
import cz.nic.tablexia.util.ui.ViewPager;

/**
 * Representing one page in ViewPager with specific game
 * Created by lhoracek on 3/4/15.
 */
public class GameMenuPage extends MenuPage implements ViewPager.ScrollListener {
    private GameDefinition game;

    private float diffEasyX, diffMediumX, diffHardX;
    private Actor diffEasy, diffMedium, diffHard;
    private Stack diffStack;
    private GameDifficulty gameDifficulty = GameDifficulty.EASY;

    private float scrollOffset = TablexiaSettings.getWorldWidth(); // hack to keep paralax layers out of picture before scrolled for first time

    public GameMenuPage(AbstractTablexiaScreen screen, GameDefinition game) {
        super(screen);
        this.game = game;

        // Title
        Texture title = getScreen().getTexture(GameMenuAssets.getResourcePath(game, GameMenuAssets.GameMenuLayers.TITLE));
        Image titleImage = new Image(title);
        int titleX = (int) (screen.getStage().getWidth() * 0.2);
        int titleWidth = (int) (screen.getStage().getWidth() * 0.6);
        int titleHeight = (int) (titleWidth * ((float) title.getHeight() / (float) title.getWidth()));
        int titleY = (int) (screen.getStage().getHeight() - (screen.getStage().getHeight() * 0.05) - titleHeight);
        titleImage.setPosition(titleX, titleY);
        titleImage.setSize(titleWidth, titleHeight);
        addActor(titleImage);

        // Start button
        Texture start = getScreen().getTexture(GameMenuAssets.getResourcePath(game, GameMenuAssets.GameMenuLayers.STARTBUTTON));
        final Image startButton = new Image(start);
        int startY = (int) (screen.getStage().getWidth() * 0.1);
        int startHeight = (int) (screen.getStage().getHeight() * 0.25);
        int startWidth = (int) (startHeight * ((float) start.getWidth() / (float) start.getHeight()));
        int startX = (int) screen.getStage().getWidth() / 2 - startWidth / 2;
        startButton.setSize(startWidth, startHeight);
        startButton.setPosition(startX, startY);

        // Start button down
        Texture startDown = getScreen().getTexture(GameMenuAssets.getResourcePath(game, GameMenuAssets.GameMenuLayers.STARTBUTTON_PRESSED));
        final Image startDownButton = new Image(startDown);
        int startDownY = (int) (screen.getStage().getWidth() * 0.1);
        int startDownHeight = (int) (screen.getStage().getHeight() * 0.25);
        int startDownWidth = (int) (startDownHeight * ((float) startDown.getWidth() / (float) startDown.getHeight()));
        int startDownX = (int) screen.getStage().getWidth() / 2 - startDownWidth / 2;
        startDownButton.setSize(startDownWidth, startDownHeight);
        startDownButton.setPosition(startDownX, startDownY);
        startDownButton.setVisible(false);

        addActor(startDownButton);
        addActor(startButton);
        startButton.addListener(new ClickListener() {
            @Override
            public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {
                startButton.addAction(new AlphaAction());
                startDownButton.setVisible(true);
                return super.touchDown(event, x, y, pointer, button);
            }

            @Override
            public void exit(InputEvent event, float x, float y, int pointer, Actor toActor) {
                AlphaAction aa = new AlphaAction();
                aa.setAlpha(1);
                startButton.addAction(aa);
                startDownButton.setVisible(false);
                super.exit(event, x, y, pointer, toActor);
            }

            @Override
            public void clicked(InputEvent event, float x, float y) {
                super.clicked(event, x, y);
                GameMenuPage.this.game.performAction(); // TODO launch game using events
            }
        });

        final Texture diff = getScreen().getTexture(GameMenuAssets.DIFF_THUMB_EASY);
        float diffY = screen.getStage().getHeight() * 0.06f;
        float diffHeight = screen.getStage().getHeight() * 0.12f;
        float diffWidth = diffHeight * ((float) diff.getWidth() / (float) diff.getHeight());

        //TODO load initial difficulty from TablexiaSettings
        // Difficulty bar
        Texture diffBar = getScreen().getTexture(GameMenuAssets.DIFF_BAR);
        Image diffBarImage = new Image(diffBar);

        int diffBarHeight = (int) (screen.getStage().getHeight() * 0.04f);
        int diffBarWidth = (int) (diffBarHeight * ((float) diffBar.getWidth() / (float) diffBar.getHeight()));
        int diffBarX = (int) screen.getStage().getWidth() / 2 - diffBarWidth / 2;
        int diffBarY = (int) (screen.getStage().getHeight() * 0.1);
        diffBarImage.setPosition(diffBarX, diffBarY);
        diffBarImage.setSize(diffBarWidth, diffBarHeight);
        addActor(diffBarImage);

        diffMediumX = (screen.getStage().getWidth() / 2) - (diffWidth / 2);
        diffEasyX = diffMediumX - (diffBarWidth / 2) + (diffWidth * 0.1f);
        diffHardX = diffMediumX + (diffBarWidth / 2) - (diffWidth * 0.1f);

        // Labels for diffuculty slider
        BitmapFont font = screen.getDefaultBoldFont();
        //font.setScale(0.5f); // TODO make text smaller
        Label.LabelStyle labelStyle = new Label.LabelStyle(font, Color.BLACK);

        Label easy = new Label(GameDifficulty.EASY.getTextDescription(), labelStyle);
        Label medium = new Label(GameDifficulty.MEDIUM.getTextDescription(), labelStyle);
        Label hard = new Label(GameDifficulty.HARD.getTextDescription(), labelStyle);

        float labelY = screen.getStage().getHeight() * 0.06f;
        float labelAddX = diffWidth / 2;
        easy.setPosition(diffEasyX + labelAddX - easy.getWidth() / 2, labelY);
        medium.setPosition(diffMediumX + labelAddX - medium.getWidth() / 2, labelY);
        hard.setPosition(diffHardX + labelAddX - hard.getWidth() / 2, labelY);

        addActor(easy);
        addActor(medium);
        addActor(hard);

        // Difficulty button
        diffStack = new Stack();
        diffStack.addActor(diffEasy = new Image(diff));
        diffStack.addActor(diffMedium = new Image(getScreen().getTexture(GameMenuAssets.DIFF_THUMB_MEDIUM)));
        diffStack.addActor(diffHard = new Image(getScreen().getTexture(GameMenuAssets.DIFF_THUMB_HARD)));

        float diffX = diffEasyX;
        diffStack.setSize(diffWidth, diffHeight);
        diffStack.setPosition(diffX, diffY);
        addActor(diffStack);
        setDifficulty(gameDifficulty);

        diffStack.addListener(new InputListener() {
            float lastX;

            @Override
            public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {
                super.touchDown(event, x, y, pointer, button);
                lastX = x;
                event.stop();
                return true;
            }

            @Override
            public void touchDragged(InputEvent event, float x, float y, int pointer) {
                super.touchDragged(event, x, y, pointer);
                float bx = diffStack.getX() + (x - lastX);
                if (bx >= diffEasyX && bx <= diffHardX) {
                    diffStack.setPosition(bx, diffStack.getY());
                }
                showDifficulty(getDifficulty());
                event.stop();
            }

            @Override
            public void touchUp(InputEvent event, float x, float y, int pointer, int button) {
                super.touchUp(event, x, y, pointer, button);
                float bx = diffStack.getX() + (x - lastX);

                GameDifficulty newDiff = getDifficulty();
                setDifficulty(newDiff);
                if (gameDifficulty != newDiff) {
                    difficultyChanged(newDiff);
                }
                event.stop();
            }
        });
    }

    /**
     * Change selector to custom difficulty
     *
     * @param difficulty
     */
    public void setDifficulty(GameDifficulty difficulty) {
        showDifficulty(difficulty);
        MoveToAction ma = new MoveToAction();
        switch (difficulty) {
            case EASY:
                ma.setPosition(diffEasyX, diffStack.getY());
                break;
            case MEDIUM:
                ma.setPosition(diffMediumX, diffStack.getY());
                break;
            case HARD:
                ma.setPosition(diffHardX, diffStack.getY());
                break;
        }
        diffStack.addAction(ma);
        if (gameDifficulty != difficulty) {
            difficultyChanged(difficulty);
        }
    }

    /**
     * Called when difficulty is changed
     *
     * @param difficulty
     */
    public void difficultyChanged(GameDifficulty difficulty) {
        gameDifficulty = difficulty;
        TablexiaSettings.getInstance().setGameDifficulty(game, difficulty);
    }

    /**
     * Get nearest difficulty from current slider position
     *
     * @return
     */
    private GameDifficulty getDifficulty() {
        float bx = diffStack.getX();
        if (bx < diffMediumX) {
            if ((diffEasyX + ((diffMediumX - diffEasyX) / 2)) > bx) {
                return GameDifficulty.EASY;
            } else {
                return GameDifficulty.MEDIUM;
            }
        } else {
            if ((diffMediumX + ((diffHardX - diffMediumX) / 2)) > bx) {
                return GameDifficulty.MEDIUM;
            } else {
                return GameDifficulty.HARD;
            }
        }
    }

    /**
     * Check difficulty image and display corresponding image
     *
     * @param diff
     */
    private void showDifficulty(GameDifficulty diff) {
        diffEasy.setVisible(false);
        diffMedium.setVisible(false);
        diffHard.setVisible(false);

        switch (diff) {
            case EASY:
                diffEasy.setVisible(true);
                break;
            case MEDIUM:
                diffMedium.setVisible(true);
                break;
            case HARD:
                diffHard.setVisible(true);
                break;
        }
    }

    @Override
    public void draw(Batch batch, float parentAlpha) {
        // Paralax layers
        // Back layer travels half the speed of scroll
        // needed to make fade work
        Color color = new Color(this.getColor().r, this.getColor().g, this.getColor().b, this.getColor().a * parentAlpha);
        batch.setColor(color);

        Texture back = getScreen().getTexture(GameMenuAssets.getResourcePath(game, GameMenuAssets.GameMenuLayers.BACK));
        float scrollHalf = (Math.abs(scrollOffset / 2) / getWidth()) * back.getWidth();
        int srcX = (int) scrollHalf;
        int srcWidth = (int) (back.getWidth() - (scrollHalf * 2));
        float x = getX() + Math.abs(Math.min(scrollOffset, 0));
        float width = getWidth() - Math.abs(scrollOffset);
        batch.draw(back, x, getY(), width, getHeight(), srcX, 0, srcWidth, back.getHeight(), false, false);

        // Mid layer travels the same speed
        batch.draw(getScreen().getTexture(GameMenuAssets.getResourcePath(game, GameMenuAssets.GameMenuLayers.MID)), getX(), getY(), getWidth(), getHeight());

        // Foreground layer travels faster and could be wider than screen. Needs to be clipped
        Texture fore = getScreen().getTexture(GameMenuAssets.getResourcePath(game, GameMenuAssets.GameMenuLayers.FORE));
        float foreWidth = (getHeight() / fore.getHeight()) * fore.getWidth();
        batch.draw(fore, getX() - ((foreWidth - getWidth()) / 2) + (scrollOffset / 2), getY(), foreWidth, getHeight());

        super.draw(batch, parentAlpha);
    }

    @Override
    public void onScroll(float offset) {
        scrollOffset = offset;
        //Log.info(((Object) this).getClass().getName(), "Scroll " + game.name() + ": " + offset);
    }

    @Override
    public void onScrolledToPage() {
        //Log.info(((Object) this).getClass().getName(), "Scrolled to page: " + game.name());
    }
}
