package cz.nic.tablexia.screen.gamemenu.pages;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Pixmap;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.InputListener;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Stack;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;

import cz.nic.tablexia.Tablexia;
import cz.nic.tablexia.TablexiaApplication;
import cz.nic.tablexia.bus.ApplicationBus;
import cz.nic.tablexia.menu.MainMenuDefinition;
import cz.nic.tablexia.screen.AbstractTablexiaScreen;
import cz.nic.tablexia.screen.encyclopedia.EncyclopediaScreen;
import cz.nic.tablexia.screen.gamemenu.GameMenuAssets;
import cz.nic.tablexia.screen.halloffame.HallOfFameScreen;
import cz.nic.tablexia.screen.profile.ProfileScreen;
import cz.nic.tablexia.screen.statistics.StatisticsScreen;
import cz.nic.tablexia.util.Log;

/**
 * Created by lhoracek on 3/4/15.
 */
public class OfficeMenuPage extends MenuPage {

    public static final Color STREET_COLOR       = Color.YELLOW;
    public static final Color HALLOFFAME_COLOR   = Color.GREEN;
    public static final Color STATISTICS_COLOR   = Color.RED;
    public static final Color ENCYCLOPEDIA_COLOR = Color.BLACK;
    public static final Color PROFILE_COLOR      = Color.BLUE;

    private Group help;

    private Image hallOfFame;
    private Image statistics;
    private Image encyclopedia;
    private Image street;
    private Image profile;


    public OfficeMenuPage(AbstractTablexiaScreen screen) {
        super(screen);
        createImageStack();

        // TODO show help layer button

        addListener(new InputListener() {
            private Color downColor;

            @Override
            public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {
                super.touchDown(event, x, y, pointer, button);
                Color color = getTouchedColor(x, y);
                downColor = color;
                help.addAction(Actions.fadeOut(0.5f));
                if (color.equals(ENCYCLOPEDIA_COLOR)) {
                    encyclopedia.setVisible(true);
                } else if (color.equals(STATISTICS_COLOR)) {
                    statistics.setVisible(true);
                } else if (color.equals(HALLOFFAME_COLOR)) {
                    hallOfFame.setVisible(true);
                } else if (color.equals(STREET_COLOR)) {
                    street.setVisible(true);
                } else if (color.equals(PROFILE_COLOR)) {
                    profile.setVisible(true);
                } else {
                    return false;
                }
                return true;
            }

            @Override
            public void touchUp(InputEvent event, float x, float y, int pointer, int button) {
                super.touchUp(event, x, y, pointer, button);
                Log.info(((Object) this).getClass().getName(), "Touch up ");
                hideAllActions();
            }
        });

        addListener(new ClickListener() {
            @Override
            public void clicked(InputEvent event, float x, float y) {
                super.clicked(event, x, y);
                Color color = getTouchedColor(x, y);
                if (color.equals(ENCYCLOPEDIA_COLOR)) {
                    ApplicationBus.getInstance().publishAsync(new Tablexia.ChangeScreenEvent(EncyclopediaScreen.class, TablexiaApplication.ScreenTransaction.FADE));
                } else if (color.equals(STATISTICS_COLOR)) {
                    ApplicationBus.getInstance().publishAsync(new Tablexia.ChangeScreenEvent(StatisticsScreen.class, TablexiaApplication.ScreenTransaction.FADE));
                } else if (color.equals(HALLOFFAME_COLOR)) {
                    ApplicationBus.getInstance().publishAsync(new Tablexia.ChangeScreenEvent(HallOfFameScreen.class, TablexiaApplication.ScreenTransaction.FADE));
                } else if (color.equals(STREET_COLOR)) {
                    ApplicationBus.getInstance().publishAsync(new ShowStreetEvent());
                } else if (color.equals(PROFILE_COLOR)) {
                    ApplicationBus.getInstance().publishAsync(new Tablexia.ChangeScreenEvent(ProfileScreen.class, TablexiaApplication.ScreenTransaction.FADE));
                }
                hideAllActions();
            }
        });
    }

    private void createImageStack() {
        Stack stack = new Stack();
        stack.setSize(getScreen().getStage().getWidth(), getScreen().getStage().getHeight());
        stack.setPosition(0, 0);
        addActor(stack);

        stack.addActor(createImage(GameMenuAssets.OFFICE));
        stack.addActor(street = createImage(GameMenuAssets.DOOR_PRESSED));
        stack.addActor(encyclopedia = createImage(GameMenuAssets.ENCYCLOPEDIA_PRESSED));
        stack.addActor(statistics = createImage(GameMenuAssets.STATISTICS_PRESSED));
        stack.addActor(hallOfFame = createImage(GameMenuAssets.HALLOFFAME_PRESSED));
        stack.addActor(createImage(GameMenuAssets.DESK));
        stack.addActor(profile = createImage(GameMenuAssets.PROFILE_PRESSED));

        hideAllActions();

        help = new Stack();
        help.setSize(getScreen().getStage().getWidth(), getScreen().getStage().getHeight());
        help.setPosition(0, 0);
        stack.addActor(help);
        Image overlay = createImage(GameMenuAssets.OVERLAY);
        overlay.getColor().a = 0.5f;
        help.addActor(overlay);
        help.addActor(createImage(GameMenuAssets.OFFICE_HELP));
    }

    private void hideAllActions() {
        profile.setVisible(false);
        street.setVisible(false);
        encyclopedia.setVisible(false);
        statistics.setVisible(false);
        hallOfFame.setVisible(false);

    }

    /**
     * Helper method for creating image instances
     *
     * @param texture
     * @return
     */
    private Image createImage(String texture) {
        Image image = new Image(getScreen().getTexture(texture));
        return image;
    }

    private Color getTouchedColor(float x, float y) {
        Texture clickmap = getScreen().getTexture(GameMenuAssets.OFFICE_CLICKMAP);
        if (!clickmap.getTextureData().isPrepared()) {
            clickmap.getTextureData().prepare();
        }

        Pixmap pixmap = clickmap.getTextureData().consumePixmap();
        int clickX = (int) (x / getWidth() * clickmap.getWidth());
        int clickY = clickmap.getHeight() - (int) (y / getHeight() * clickmap.getHeight());
        return new Color(pixmap.getPixel(clickX, clickY));
    }

    /**
     * Event fired by clicking on door telling viewpager to page to first game
     */
    public static class ShowStreetEvent implements ApplicationBus.ApplicationEvent {
    }
}
