package cz.nic.tablexia.screen.preferences;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.ui.Container;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.List;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.scenes.scene2d.ui.SelectBox;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;
import com.badlogic.gdx.scenes.scene2d.utils.TextureRegionDrawable;

import cz.nic.tablexia.TablexiaSettings;
import cz.nic.tablexia.graphics.ColorTexture;
import cz.nic.tablexia.loader.application.ApplicationTextureManager;
import cz.nic.tablexia.screen.AbstractTablexiaScreen;

public class PreferencesScreen extends AbstractTablexiaScreen<Void> {

    private static final String BACKGROUND_TEXTURE  = ApplicationTextureManager.UniversalAssets.BACKGROUND_WOODEN;
    public static final String  TEXT_RESOURCES_FILE = "text/application/preferences";
    public static final String  LANGUAGE_LABEL_KEY  = "language_label";

    private Container<Table> preferencesContainer;

    @Override
    protected void prepareScreenTextureAssetNames(java.util.List<String> textureFileNames) {
        textureFileNames.add(BACKGROUND_TEXTURE);
    }

    @Override
    protected String prepareScreenTextResourcesAssetName() {
        return TEXT_RESOURCES_FILE;
    }

    @Override
    protected void screenLoaded() {
        BitmapFont          font        = getDefaultRegularFont();
        Label.LabelStyle    labelStyle  = new Label.LabelStyle(font, Color.WHITE);

        preferencesContainer = new Container<Table>();
        preferencesContainer.setBackground(new TextureRegionDrawable(new TextureRegion(getTexture(BACKGROUND_TEXTURE))));
        getStage().addActor(preferencesContainer);

        Table preferencesLayout = new Table();
        preferencesContainer.setActor(preferencesLayout);
        preferencesContainer.left().top();
        preferencesContainer.pad(getStage().getWidth() / 10);
        preferencesContainer.setBounds(0, 0, getStage().getWidth(), getStage().getHeight());

        // Languages Label
        Label languagesLabel = new Label(getText(LANGUAGE_LABEL_KEY), labelStyle);
        preferencesLayout.add(languagesLabel);

        // Languages SelectBox
        ScrollPane.ScrollPaneStyle scrollPaneStyle = new ScrollPane.ScrollPaneStyle();
        scrollPaneStyle.background = new TextureRegionDrawable(new TextureRegion(new ColorTexture(50, 50, Color.GRAY)));
        List.ListStyle listStyle = new List.ListStyle();
        listStyle.selection = new TextureRegionDrawable(new TextureRegion(new ColorTexture(50, 50, Color.GRAY)));
        listStyle.font = font;
        SelectBox.SelectBoxStyle selectBoxStyle = new SelectBox.SelectBoxStyle();
        selectBoxStyle.font = font;
        selectBoxStyle.scrollStyle = scrollPaneStyle;
        selectBoxStyle.listStyle = listStyle;
        selectBoxStyle.background = new TextureRegionDrawable(new TextureRegion(new ColorTexture(50, 50, Color.GRAY)));
        SelectBox<TablexiaSettings.LocaleDefinition> languagesSelectBox = new SelectBox<TablexiaSettings.LocaleDefinition>(selectBoxStyle);
        languagesSelectBox.setItems(TablexiaSettings.LocaleDefinition.values());
        languagesSelectBox.setSelected(TablexiaSettings.getInstance().getLocaleDefinition());
        languagesSelectBox.addCaptureListener(new ChangeListener() {

            @Override
            public void changed(ChangeEvent event, Actor actor) {
                TablexiaSettings.getInstance().setLocale(((SelectBox<TablexiaSettings.LocaleDefinition>) event.getTarget()).getSelected());
            }
        });
        preferencesLayout.add(languagesSelectBox).expandX().left();
    }

    @Override
    public void screenResized(int width, int height) {
        preferencesContainer.setBounds(0, 0, getStage().getWidth(), getStage().getHeight());
    }
}
