package cz.nic.tablexia.util.ui;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.utils.viewport.Viewport;

import cz.nic.tablexia.TablexiaSettings;
import cz.nic.tablexia.util.Log;

/**
 * Created by lhoracek on 3/6/15.
 */
public class XFillViewport extends Viewport {

    /**
     * Creates a new viewport using a new {@link com.badlogic.gdx.graphics.OrthographicCamera} with no maximum world size.
     */
    public XFillViewport() {
        setWorldSize(TablexiaSettings.getWorldWidth(), TablexiaSettings.getMinWorldHeight());
        setCamera(new OrthographicCamera());
        Log.info(getClass().getName(), "Viewport size: " + TablexiaSettings.getWorldWidth() + "x" + TablexiaSettings.getMinWorldHeight());
    }

    private float cameraYOffset;

    @Override
    public void update(int screenWidth, int screenHeight, boolean centerCamera) {
        float newWorldHeight = getWorldWidth() / ((float) screenWidth / (float) screenHeight);

        // FIT into screen keeping at least minimum ration
        //int screenX = (newWorldHeight < TablexiaSettings.getMinWorldHeight()) ? ((int) ((screenWidth - (screenHeight / TablexiaSettings.getMaximumRatio())) / 2)) : 0;
        int screenX = 0;
        int screenY = 0;
        //int newScreenWidth = (newWorldHeight < TablexiaSettings.getMinWorldHeight()) ? ((int) (screenHeight / TablexiaSettings.getMaximumRatio())) : screenWidth;
        int newScreenWidth = screenWidth;
        int newScreenHeight = screenHeight;

        // SET NEW WORLD HEIGHT
        this.setWorldHeight(Math.max(TablexiaSettings.getMinWorldHeight(), newWorldHeight)); // World width dooes not change
        this.setWorldWidth(TablexiaSettings.getWorldWidth());
        cameraYOffset = (getWorldHeight() - TablexiaSettings.getMinWorldHeight()) / 2;
        setScreenBounds(screenX, screenY, newScreenWidth, newScreenHeight);
        apply(centerCamera);
    }

    @Override
    public void apply(boolean centerCamera) {
        Log.info(getClass().getName(), "New viewport size (min: " + TablexiaSettings.getMinWorldHeight() + ") position: " + getScreenX() + ":" + getScreenY() + " - bounds: " + getScreenWidth() + ":" + getScreenHeight() + " - world: " + getWorldWidth() + "x" + getWorldHeight() + " - y offset: " + cameraYOffset);

        Gdx.gl.glViewport(getScreenX(), getScreenY(), getScreenWidth(), getScreenHeight());
        getCamera().viewportWidth = getWorldWidth();
        getCamera().viewportHeight = getWorldHeight();
        if (centerCamera) getCamera().position.set(getWorldWidth() / 2, getWorldHeight() / 2, 0);
        getCamera().position.y -= cameraYOffset;
        getCamera().update();
    }

}