package cz.nic.tablexia.util.ui.dialog;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.TextureAtlas;
import com.badlogic.gdx.math.Interpolation;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Dialog;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.utils.NinePatchDrawable;

import cz.nic.tablexia.loader.application.ApplicationFontManager;
import cz.nic.tablexia.util.ui.dialog.text.DialogTextContent;

import static com.badlogic.gdx.scenes.scene2d.actions.Actions.sequence;

/**
 * Created by Václav Tarantík on 17.3.15.
 */
public class TablexiaDialog extends Dialog {

    public static TextureAtlas backgroundAtlas;
    public static TextureAtlas buttonAtlas;

    protected DialogTextContent dialogTextContent;

    public enum BackGroundType {
        BUBBLE_CLASSIC("bubble_classic"),
        BUBBLE_ARROW_DOWN("bubble_arrow_down"),
        BUBBLE_ARROW_LEFT("bubble_arrow_left"),
        BUBBLE_CLASSIC_CONTINUE_BUTTON("bubble_classic_continue_button"),
        DIALOG_RECTANGLE("dialog_rectangle"),
        DIALOG_SQUARE("dialog_square"),
        DIALOG_SQUARE_BORDERLINES("dialog_square_borderlines");

        private String dialogBackgroundTextureName;

        BackGroundType(String dialogBackgroundTextureName){
            this.dialogBackgroundTextureName = dialogBackgroundTextureName;
        }
    }

    public enum ButtonType{
        BLUE("tablexiabutton_blank_blue"),
        GREEN("tablexiabutton_blank_green"),
        RED("tablexiabutton_blank_red");

        private String buttonResourceName;

        ButtonType(String buttonResourceName){
            this.buttonResourceName = buttonResourceName;
        }

        public String getButtonResourceName(){
            return buttonResourceName;
        }
    }

    static {
        backgroundAtlas = new TextureAtlas(Gdx.files.internal("atlases/dialog_background.atlas"));
        buttonAtlas = new TextureAtlas(Gdx.files.internal("atlases/buttons_atlas.pack"));
    }

    public TablexiaDialog(float x, float y,float width,float height, BackGroundType backGroundType) {
        super("", new WindowStyle(ApplicationFontManager.getInstance().getDefaultApplicationRegularFont(),
                                  Color.BLACK,
                                  new NinePatchDrawable(backgroundAtlas.createPatch(backGroundType.dialogBackgroundTextureName))));

        setModal(true);
        setMovable(false);
        setResizable(false);

        setBounds(x, y, width, height);
        getBackground().setMinWidth(width);
        getBackground().setMinHeight(height);
    }

    @Override
    public Dialog show(Stage stage) {
        prepareContent();
        show(stage, sequence(Actions.alpha(0), Actions.fadeIn(0.4f, Interpolation.fade)));
        return this;
    }

    protected void prepareContent(){
        //no content here
    }
}